#! /bin/tcsh -f
#
#
#
#

set pwd = `pwd`
set cpus = 4

if("$1" == "sumup") goto sumup

if("$1" == "clean") then
    killall runme_cpu.com
    killall fastBragg
    rm -f cpu*/n.txt
    rm -f cpu*/sum.bin
    rm -rf /dev/shm/${USER}/cpu*
#    ./runme_multi.com
endif

if(! -e seeds.txt) then
    awk 'BEGIN{for(i=1;i<2e6;++i)printf("%d\n",i)}' >! seeds.txt
endif
if(! -e noisify) cp ../noisify .


foreach cpu ( `seq 1 $cpus` )

    cd $pwd

    rm -f cpu$cpu
    mkdir -p /dev/shm/${USER}/cpu$cpu
    ln -sf /dev/shm/${USER}/cpu$cpu cpu${cpu}

    awk -v cpu=$cpu -v cpus=$cpus 'NR%cpus==(cpu-1)' seeds.txt >! cpu${cpu}/seeds.txt
    cp P1.hkl UBtoA.awk fastBragg cpu${cpu}/
    

    cd cpu${cpu}/
      ${pwd}/runme_cpu.com $* >! runme.log &

    cd $pwd

end

sumup:

set started = 0
while ( $started < $cpus )
    sleep 3
    set started = `ls -l cpu*/ | grep sum.bin | wc -l`
    echo "started: $started"
end

set N = 80

while ( 1 )

set n = 0
float_add cpu1/sum.bin -scale1 0 cpu1/sum.bin -scale2 0 sum.bin >! /dev/null
foreach file ( cpu*/sum.bin )
    set cpudir = `echo $file | awk -F "/" '{print $1}'`
    @ n = ( $n + `cat ${cpudir}/n.txt` )
    float_add sum.bin $file
    mv output.bin sum.bin
    echo $n
end

set norm = `echo $n | awk '{print 1/$1}'`
float_add sum.bin -scale1 $norm sum.bin -scale2 0 avg.bin | tee tempfile.txt
set scale = `awk '/^mean/{print 100/$3}' tempfile.txt`

set dist = `head -512c cpu1/intimage.img | awk -F "=" '/^DIST/{print $2+0}'`
set wave = `head -512c cpu1/intimage.img | awk -F "=" '/^WAVE/{print $2+0}'`
set pix = `head -512c cpu1/intimage.img | awk -F "=" '/^PIXEL_SIZE/{print $2+0}'`
set sizex = `head -512c cpu1/intimage.img | awk -F "=" '/^SIZE1/{print $2+0}'`
set sizey = `head -512c cpu1/intimage.img | awk -F "=" '/^SIZE2/{print $2+0}'`
set beamx = `head -512c cpu1/intimage.img | awk -F "=" '/^BEAM_CENTER_X/{print $2+0}'`
set beamy = `head -512c cpu1/intimage.img | awk -F "=" '/^BEAM_CENTER_Y/{print $2+0}'`
set detsize_x = `echo $pix $sizex | awk '{print $1*$2}'`
set detsize_y = `echo $pix $sizey | awk '{print $1*$2}'`
set Xbeam = $beamx
set Ybeam = `echo $detsize_y $beamx $beamy | awk '{print $1-$3}'`

./noisify -floatfile avg.bin -distance $dist -lambda $wave \
  -detsize_x $detsize_x -detsize_y $detsize_y -pixel $pix \
  -Xbeam $Xbeam -Ybeam $Ybeam -scale $scale 


cp avg.bin dumpme.bin
od -v -f -w4 dumpme.bin |\
awk -v edge=$sizex -v pix=$pix -v Xbeam=$Xbeam -v Ybeam=$Ybeam '\
  NF<2{next} x>=edge{++y;x=0} {print x*pix-Xbeam,y*pix-Ybeam,$2; ++x}' |\
awk '{print sqrt($1*$1+$2*$2),$3}' |\
sort -g |\
tee I_vs_R.txt |\
tail

cat I_vs_R.txt |\
awk -v pix=$pix -v dist=$dist -v lambda=$wave '{R=$1;I=$2;\
   airpath=sqrt(R*R+dist*dist);sin2theta=R/airpath;cos2theta=dist/airpath;\
   twotheta=atan2(sin2theta,cos2theta);theta=twotheta/2;\
   stol=sin(theta)/lambda;\
   omega=pix*pix/airpath/airpath*dist/airpath;\
   polar=0.5*(1+cos(2*theta)^2);\
   print stol,I/polar/omega}' |\
cat >! Inorm_vs_stol.txt

cat Inorm_vs_stol.txt |\
awk -v bs=5e-5 '{stol=$1;I=$2;\
   bin=sprintf("%.0f",stol/bs)*bs;sum[bin]+=$2;++count[bin]}\
END{for(bin in sum) print bin,sum[bin]/count[bin]}' |\
sort -g |\
cat >! Iavg_vs_stol.txt


cat Iavg_vs_stol.txt |\
awk -v lambda=$wave -v w=10 '{++n;stol[n]=$1;I[n]=$2} END{\
  for(i=1;i<=n;++i){\
     count=sum=0;for(j=-w;j<w;++j){\
        if(I[i+j]!=""){++count;sum+=I[i+j]}\
     };\
     print stol[i],sum/count;\
  }\
}' |\
cat >! Iint_vs_stol.txt


cat Iavg_vs_stol.txt |\
awk -v lambda=$wave '{stol=$1;I=$2;\
  sintheta=stol*lambda;costheta=sqrt(1-sintheta**2);theta=atan2(sintheta,costheta);\
  print stol,I*sin(theta)*sin(2*theta)}' |\
cat >! Icorr_vs_stol.txt



echo $n | tee n.txt

sleep 300

end
